require "behaviours/wander"
require "behaviours/attackwall"
require "behaviours/minperiod"
require "behaviours/leash"
require "behaviours/doaction"
require "behaviours/follow"

local choppybrain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
end)

local MIN_FOLLOW = 4
local MAX_FOLLOW = 11
local MED_FOLLOW = 6
local MAX_WANDER_DIST = 10
local RUN_AWAY_DIST = 5
local GO_HOME_DIST = 5
local MAX_CHASE_TIME = 6

function choppybrain:OnAttacked(attacker)
    self.inst.components.combat:SetTarget(attacker)
    self.inst.components.combat:ShareTarget(attacker, 30, function(dude) return dude:HasTag("summonedbyplayer") and dude.components.follower.leader == GetPlayer() end, 5)
end

function choppybrain:OnAttackOther(target)
    self.inst.components.combat:ShareTarget(target, 30, function(dude) return dude:HasTag("summonedbyplayer") and dude.components.follower.leader == GetPlayer() end, 5)
end

local function EatFoodAction(inst)
    local target = FindEntity(inst, SEE_DIST, function(item) return inst.components.eater:CanEat(item) and item:IsOnValidGround() end)
    if target then
        return BufferedAction(inst, target, ACTIONS.EAT)
    end
end

local function EatGemAction(inst)
    local target = FindEntity(inst, SEE_DIST, function(item) return (item.prefab == "bluegem" or item.prefab == "redgem") and item:IsOnValidGround() end)
    if target then
        return BufferedAction(inst, target, ACTIONS.PICKUP)
    end
end

local function GetLeader(inst)
    return inst.components.follower and inst.components.follower.leader
end

local function GetStayPos(inst)
	return inst.components.followersitcommand.locations["currentstaylocation"]
end


local function GetWanderPoint(inst)
	if inst.components.followersitcommand and inst.components.followersitcommand:IsCurrentlyStaying() then
		return GetStayPos(inst)
	else
		local target = GetLeader(inst) or GetPlayer()
   		if target then
        		return target:GetPosition()
    		end

	end
end


local function ShouldGoHome(inst)
    local homePos = inst.components.followersitcommand.locations["currentstaylocation"]
    local myPos = Vector3(inst.Transform:GetWorldPosition() )
    return (homePos and distsq(homePos, myPos) > GO_HOME_DIST*GO_HOME_DIST)
end

local function GoHomeAction(inst)
    local homePos = inst.components.followersitcommand.locations["currentstaylocation"]
    if homePos then
        return BufferedAction(inst, nil, ACTIONS.WALKTO, nil, homePos, nil, 0.2)
    end
end
 
local function GetFaceTargetFn(inst)
    return inst.components.follower.leader
end

local function KeepFaceTargetFn(inst, target)
    return inst.components.follower.leader == target
end 
			 
function choppybrain:OnStart()
    
    local root = PriorityNode(
    {
		ChaseAndAttack(self.inst, MAX_CHASE_TIME),
		Follow(self.inst, function() return self.inst.components.follower.leader end, MIN_FOLLOW, MED_FOLLOW, MAX_FOLLOW, true),
		--FaceEntity(self.inst, GetFaceTargetFn, KeepFaceTargetFn),
        Wander(self.inst, function() return Point(GetPlayer().Transform:GetWorldPosition()) end , MAX_WANDER_DIST)        
    }, .5)
        
    self.bt = BT(self.inst, root)

    self.inst:ListenForEvent("attacked", function(inst, data) self:OnAttacked(data.attacker) end)
    self.inst:ListenForEvent("onattackother", function(inst, data) self:OnAttackOther(data.target) end)
	
end

return choppybrain

