

local brain = require "brains/choppybrain"

local assets =
{
	Asset("ATLAS", "images/inventoryimages/choppy.xml"),
		Asset("ANIM", "anim/hound_basic.zip"),
	Asset("ANIM", "anim/hound.zip"),
	Asset("ANIM", "anim/hound_red.zip"),
	Asset("ANIM", "anim/hound_ice.zip"),
	Asset("SOUND", "sound/hound.fsb"),
}

local prefabs =
{
	"meat",
	"log", 
	"character_fire",
    "livinglog",
    "redgem",
    "bluegem",
}


local function ondeath(inst)
	inst.components.sanityaura.penalty = 0
	local player = GetPlayer()
	if player then
		player.components.sanity:RecalculatePenalty()
	end
end


local function NormalRetargetFn(inst)

    local newtarget = FindEntity(inst, 20, function(guy)
            return  guy.components.combat and 
                    inst.components.combat:CanTarget(guy) and
                    (guy.components.combat.target == GetPlayer() or GetPlayer().components.combat.target == guy)
    end)

    return newtarget
end

local function OnAttacked(inst, data)
    --print(inst, "OnAttacked")
    local attacker = data.attacker

    if attacker and attacker:HasTag("player") then
        inst.components.health:SetVal(0)
    else
        inst.components.combat:SetTarget(attacker)
    end
end

local function auratest(inst, target)

    if target == GetPlayer() then return false end

    local leader = inst.components.follower.leader
    if target.components.combat.target and ( target.components.combat.target == inst or target.components.combat.target == leader) then return true end
    if inst.components.combat.target == target then return true end

    if leader then
        if leader == target then return false end
        if target.components.follower and target.components.follower.leader == leader then return false end
    end

    return target:HasTag("monster") or target:HasTag("prey")
end


local function ShouldAcceptItem(inst, item)
	if (item.prefab == "bluegem" or item.prefab == "redgem") and inst.components.houndtypes.currenttype =="normal" then
        	return true
   	end
end

local function OnGetItemFromPlayer(inst, giver, item)
    	if item.prefab == "bluegem" then
		
		inst.components.houndtypes:MakeMeBlue()
	elseif item.components.edible then
                inst.sg:GoToState("eat")
		inst.components.houndtypes:MakeMeRed()
    	end
end

local function OnRefuseItem(inst, item)
    inst.sg:GoToState("taunt")
    local playerprefab = GetPlayer()
    if playerprefab then
	    playerprefab.components.talker:Say(GetString(playerprefab.prefab, "ACTIONFAIL_GENERIC"))
    end
end


local function fn(Sim)


    
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local sound = inst.entity:AddSoundEmitter()
	local shadow = inst.entity:AddDynamicShadow()
	local whoisplayer = GetPlayer()

	shadow:SetSize( 4, 1.5 )
    inst.Transform:SetFourFaced()
	inst:AddTag("epic")
	
	MakeCharacterPhysics(inst, 1000, .5)

    --inst:AddTag("monster")
	inst:AddTag("character")
    inst:AddTag("choppy")
    inst:AddTag("scarytoprey")
    inst:AddTag("summonedbyplayer")
    inst:AddTag("noauradamage")
    inst:AddTag("notraptrigger")

    anim:SetBank("hound")
    anim:SetBuild("hound")
    anim:PlayAnimation("idle")
    
    ------------------------------------------

    inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
    inst.components.locomotor.runspeed = 14
    inst.components.locomotor.walkspeed = 14
    
    ------------------------------------------
    inst:SetStateGraph("SGchoppy")

    ------------------------------------------

    MakeHugeFreezableCharacter(inst, "marker")
    ------------------
    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(200)
    ------------------  
    inst:AddComponent("combat")
	inst.components.combat.radius = 3
    inst.components.combat:SetDefaultDamage(20)
    inst.components.combat.hiteffectsymbol = "marker"
    inst.components.combat:SetAttackPeriod(TUNING.LEIF_ATTACK_PERIOD/200)
     inst.components.combat:SetRetargetFunction(3, NormalRetargetFn)
   
	
	inst:AddComponent("sanityaura")
    inst.components.sanityaura.penalty = TUNING.SHADOWWAXWELL_SANITY_PENALTY
	
    
    ------------------------------------------
    inst.entity:AddLabel()
    inst.Label:SetFontSize(25)
    inst.Label:SetFont(DEFAULTFONT)
    inst.Label:SetPos(0,6,0)
    inst.Label:SetColour(1, 1, 1)
    inst.Label:Enable(true)
    ------------------------------------------

    inst:AddComponent("follower")
    inst.components.follower.maxfollowtime = 99999999
    --print("treeguard should follow "..whoisplayer)
    whoisplayer.components.leader:AddFollower(inst)
    inst.components.follower:AddLoyaltyTime(9999999)
    ------------------------------------------
    inst:AddComponent("inspectable")
    local minimap = inst.entity:AddMiniMapEntity()
    minimap:SetIcon( "sign.png" )
    ------------------------------------------
    inst:AddComponent("eater")
    inst.components.eater:SetCarnivore()
    inst.components.eater:SetCanEatHorrible()

    inst.components.eater.strongstomach = true -- can eat monster meat!
    ------------------------------------------
    inst.Transform:SetScale(0.6, 0.6, 0.6)  
    
    inst:SetBrain(brain)


    inst:AddComponent("named")
    inst.components.named.possiblenames = STRINGS.CHOPPY_NAMES
    inst.components.named:PickNewName()
    
    if ACTIONS.SITCOMMAND ~= nil then
    	inst:AddComponent("followersitcommand")
    end

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetLoot({"necronomicon"})

    inst:AddComponent("houndtypes")

     inst:AddComponent("trader")
    inst.components.trader:SetAcceptTest(ShouldAcceptItem)
    inst.components.trader.onaccept = OnGetItemFromPlayer
    inst.components.trader.onrefuse = OnRefuseItem

    return inst
end

return Prefab( "common/choppy", fn, assets, prefabs)

