local assets =
{
	Asset("ANIM", "anim/necronomicon.zip"),	
	
    Asset("ATLAS", "images/inventoryimages/necronomicon.xml"),
    Asset("IMAGE", "images/inventoryimages/necronomicon.tex"),
}

local prefabs =
{
	"choppy"
}

local function doeffects(inst, pos)
    SpawnPrefab("statue_transition").Transform:SetPosition(pos:Get())
    SpawnPrefab("statue_transition_2").Transform:SetPosition(pos:Get())
end

local function canread(inst)
    return (inst.components.sanity:GetMaxSanity() >= TUNING.SHADOWWAXWELL_SANITY_PENALTY)
end

local function onread(inst, reader)
    GetPlayer().components.sanity:DoDelta(-TUNING.SANITY_HUGE)
    local choppy = SpawnPrefab("choppy")
    choppy.Transform:SetPosition(inst.Transform:GetWorldPosition())
    inst.SoundEmitter:PlaySound("dontstarve/maxwell/shadowmax_appear")
    inst:Remove()
	
    --Check sanity
    if not canread(reader) then 
        if reader.components.talker then
            reader.components.talker:Say(GetString(reader.prefab, "ANNOUNCE_NOSANITY"))
            return true
        end
    end

    --Check reagent
    if not reader.components.inventory:Has("necronomicon", TUNING.SHADOWWAXWELL_FUEL_COST) then
        if reader.components.talker then
            reader.components.talker:Say(GetString(reader.prefab, "NOFUEL"))
            return true
        end
    end
	
	reader.components.inventory:ConsumeByName("necronomicon", TUNING.SHADOWWAXWELL_FUEL_COST)
	
    --Ok you had everything. Make the image.
    local theta = math.random() * 2 * PI
    local pt = inst:GetPosition()
    local radius = math.random(3, 6)
    local offset = FindWalkableOffset(pt, theta, radius, 12, true)
    if offset then
        local image = SpawnPrefab("choppy")
        local pos = pt + offset
        image.Transform:SetPosition(pos:Get())
        doeffects(inst, pos)
        image.components.follower:SetLeader(reader)
        reader.components.sanity:RecalculatePenalty()
        inst.SoundEmitter:PlaySound("dontstarve/maxwell/shadowmax_appear")
        return true
    end
end


local function fn()
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
    local sound = inst.entity:AddSoundEmitter()
		
    inst.AnimState:SetBank("bushhat")
    inst.AnimState:SetBuild("necronomicon")
    inst.AnimState:PlayAnimation("anim")
	
	MakeInventoryPhysics(inst)
    inst:AddTag("irreplaceable")
	
    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.imagename = "necronomicon"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/necronomicon.xml"
	
    inst:AddComponent("characterspecific")
    inst.components.characterspecific:SetOwner("wesna")

	-- a minimap icon must be specified
	inst.entity:AddMiniMapEntity()
	inst.MiniMapEntity:SetIcon( "necronomicon.tex" )
	
    -----------------------------------
    inst:AddComponent("inspectable")
    inst:AddComponent("book")
    inst.components.book.onread = onread


    return inst
end

return Prefab("common/inventory/necronomicon", fn, assets, prefabs)